
# q-expansion of newform 912.2.bo.f, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 912
weight = 2
poly_data = [1, 0, 0, -1, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[799, [1, 0, 0, 0, 0, 0]], [229, [1, 0, 0, 0, 0, 0]], [305, [1, 0, 0, 0, 0, 0]], [97, [0, 0, 0, 0, 0, -1]]]
aps_data = [[0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0], [1, 0, -1, 1, 0, 2], [-1, 1, -1, 1, 0, 0], [0, -1, 0, 0, 0, -1], [-1, -1, 2, 1, 0, -2], [-2, 2, -3, 3, -2, 2], [2, 1, -2, 2, 1, 2], [2, -5, 0, 3, 3, 2], [2, -5, -2, -5, 2, 0], [-3, -3, 0, 3, 3, 3], [-3, -2, -2, 0, 6, -4], [-1, 3, -4, -3, 0, 3], [1, 0, -1, -3, -5, -2], [0, 3, -1, 3, 0, 0], [-2, -6, 0, 8, 8, -1], [-6, -4, -3, 3, 4, 6], [1, 0, 0, -1, -1, 11], [-4, 1, 4, 1, -4, 0], [-7, 0, 7, 1, 5, -6], [7, -1, 5, -2, 0, 2], [5, -5, 10, 5, 0, -5], [9, -2, -1, -9, 3, 3], [-7, -7, -5, 4, 3, 5], [-1, 4, 0, 0, -4, 1], [4, 4, 1, -8, 4, -1], [0, -1, -6, -1, -6, -1], [7, 3, -7, -7, 4, 4], [-2, 0, 2, 1, -13, -1], [-4, 5, 5, 0, 0, -5], [2, 2, 8, -6, 4, -8], [3, -2, -2, 2, 2, -3], [-8, 8, 0, 0, 0, -1], [3, 3, -5, -4, 1, 5], [3, 8, -2, -5, 0, 5], [-8, 14, 14, 0, -5, -9], [2, 0, -2, -5, -8, -3], [0, 5, -4, -16, -4, 5], [-5, 5, 0, 0, 0, -4], [5, 1, 2, -2, -1, -5], [0, 8, -8, -13, -8, 8], [13, -9, 4, -9, 13, 0], [0, -4, -4, 0, 9, -5], [-4, -5, 1, 5, 0, -5], [-1, -7, -4, 1, 11, 11], [2, 5, 10, 5, 2, 0], [0, 11, -3, 3, -11, 0], [-3, 0, 3, -1, 1, -4], [3, -3, -3, 0, 5, -2], [-14, 8, 8, 0, -4, -4], [2, 0, -2, -5, -8, -3], [0, -8, 14, -4, 14, -8], [10, 10, 5, -2, -8, -5], [-2, 3, 0, -1, -1, 13], [-15, 8, 4, 8, -15, 0], [10, 1, 3, -7, 0, 7], [11, 15, 6, -5, 0, 5], [3, 0, -3, 10, -3, 13], [0, 9, -3, 11, -3, 9], [3, -3, 0, 0, 0, 5], [5, 1, -6, 6, -1, -5], [0, -5, -8, 1, -8, -5], [5, 13, 2, -3, 0, 3], [-9, 8, -5, 9, -3, -3], [13, 7, -3, 7, 13, 0], [3, 3, 5, 2, -5, -5], [0, -20, 4, -9, 4, -20], [-5, 0, 5, 16, 2, 11], [2, 0, -2, 0, -5, 2], [-5, 0, 5, 5, -5, -5], [0, -6, 0, -27, 0, -6], [4, -8, 0, 0, 8, -4], [6, 6, -6, -5, -1, 6], [13, -10, -7, -13, 17, 17], [6, 0, 0, 0, -7, 7], [0, -9, -2, -2, 0, 2], [8, 3, 1, 3, 8, 0], [11, 0, 4, -4, 0, -11], [15, 5, 17, -17, -5, -15], [-14, 6, 10, 6, -14, 0], [7, 15, 15, 0, 4, -19], [-20, -10, -13, 7, 0, -7], [-13, -13, 1, 13, 0, -1], [20, -10, 0, -10, -10, 0], [-5, -23, -8, 8, 23, 5], [11, 11, 17, -8, -3, -17], [-12, 14, -4, 12, -10, -10], [-14, -19, -19, 0, 25, -6], [-8, 0, 8, 7, -15, -1], [3, -6, 9, -3, -3, -3], [0, -6, -3, 26, -3, -6], [-4, 11, 0, -7, -7, 18], [13, 2, -11, -13, 9, 9], [4, -11, -12, -16, 0, 16], [-9, 0, 9, 24, -5, 15], [-3, 15, -1, 15, -3, 0], [-6, -21, 0, 27, 27, 1], [0, 12, 1, 4, 1, 12], [-8, 21, -3, 21, -8, 0], [19, -1, 7, -1, 19, 0], [7, -14, 0, 7, 7, 17], [-2, -2, 15, -1, 3, -15], [-5, -2, 17, 5, -15, -15], [-3, 8, 8, 0, -16, 8], [-21, -11, -11, 0, 9, 2], [26, -11, 4, -26, 7, 7], [-22, -8, -3, 3, 8, 22], [14, -3, 0, -11, -11, -11], [-11, 3, 2, 3, -11, 0], [-5, 13, -19, 5, 6, 6], [-8, -17, -17, 0, 10, 7], [7, 0, -7, 1, -16, 8], [-8, 16, 0, 16, -8, 0], [0, 6, 10, 0, 10, 6], [-7, 13, 0, -6, -6, 3], [-21, 0, 21, 16, -6, -5], [2, -29, 4, 2, 0, -2], [7, -15, -15, 0, 13, 2], [-5, 21, -5, 5, -16, -16], [-6, -6, -13, 19, -13, 13], [5, -7, 0, 2, 2, -10], [0, 25, -21, -1, -21, 25], [-14, 8, -9, 14, 1, 1], [30, -4, -4, 0, 8, -4], [5, 12, -8, -5, -4, -4], [13, 13, 20, -20, -13, -13], [-6, -6, -6, -19, 25, 6], [-5, 2, -24, -19, 0, 19], [-7, 0, 7, -10, -11, -17], [0, 17, -16, 11, -16, 17], [-2, -22, 2, -2, 22, 2], [10, 21, 5, -5, -21, -10], [-2, 0, 6, 0, -2, 0], [6, 8, -14, -20, 0, 20], [3, -1, -1, 0, 27, -26], [-14, 15, 1, 15, -14, 0], [10, 10, -5, -22, 12, 5], [0, 21, -23, 3, -23, 21], [21, -5, -5, 0, -4, 9], [0, -12, -9, -12, -9, -12], [25, 25, 5, 2, -27, -5], [-3, -16, 0, 19, 19, 25], [-11, -11, -13, 26, -15, 13], [-20, -9, 10, -9, -20, 0], [13, 0, 0, -13, 0, 0], [-8, -13, -2, 6, 0, -6], [8, -22, -6, 6, 22, -8], [12, 8, -19, 19, -8, -12], [-10, 3, 0, 7, 7, 18], [0, 16, -27, -12, -27, 16], [1, 2, 18, 17, 0, -17], [4, 9, 13, -4, -22, -22], [18, 0, -3, 0, 18, 0], [11, 11, 2, 4, -15, -2], [-8, 0, 8, 6, -9, -2], [11, 4, 4, 0, -21, 17], [14, 1, 5, -14, -6, -6], [-26, 3, 5, -5, -3, 26], [24, 24, 7, -21, -3, -7], [-4, 3, -12, 3, -4, 0], [10, 13, 23, 13, 0, -13], [-8, 6, -28, -20, 0, 20], [-10, -17, 14, -14, 17, 10], [18, -14, 28, -28, 14, -18], [0, 5, 1, -23, 1, 5], [-4, 0, 4, 7, 34, 3], [22, -1, 12, -10, 0, 10], [-17, -11, 20, -11, -17, 0]]
